"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPersistedConfig = exports.setPersistedConfig = exports.get$select = exports.restore$select = exports.hide$select = exports.checkMulti = exports.handleConditionalSelect = exports.throwUnlessCan = exports.getAbility = exports.makeDefaultOptions = exports.makeOptions = void 0;
const get_1 = __importDefault(require("lodash/get"));
const set_1 = __importDefault(require("lodash/set"));
const unset_1 = __importDefault(require("lodash/unset"));
const errors_1 = require("@feathersjs/errors");
const feathers_utils_1 = require("feathers-utils");
const getFieldsForConditions_1 = __importDefault(require("../../utils/getFieldsForConditions"));
const common_1 = require("../common");
const getAvailableFields_1 = __importDefault(require("../../utils/getAvailableFields"));
const makeOptions = (app, options) => {
    options = options || {};
    return Object.assign(common_1.makeDefaultBaseOptions(), defaultOptions, getAppOptions(app), options);
};
exports.makeOptions = makeOptions;
const defaultOptions = {
    adapter: "feathers-memory",
    availableFields: (context) => {
        var _a, _b;
        const availableFields = (_b = (_a = context.service.options) === null || _a === void 0 ? void 0 : _a.casl) === null || _b === void 0 ? void 0 : _b.availableFields;
        return getAvailableFields_1.default(context, { availableFields });
    }
};
const makeDefaultOptions = (options) => {
    return Object.assign(common_1.makeDefaultBaseOptions(), defaultOptions, options);
};
exports.makeDefaultOptions = makeDefaultOptions;
const getAppOptions = (app) => {
    const caslOptions = app === null || app === void 0 ? void 0 : app.get("casl");
    return (caslOptions && caslOptions.authorizeHook)
        ? caslOptions.authorizeHook
        : {};
};
const getAbility = (context, options) => {
    var _a, _b;
    // if params.ability is set, return it over options.ability
    if ((_a = context === null || context === void 0 ? void 0 : context.params) === null || _a === void 0 ? void 0 : _a.ability) {
        if (typeof context.params.ability === "function") {
            const ability = context.params.ability(context);
            return Promise.resolve(ability);
        }
        else {
            return Promise.resolve(context.params.ability);
        }
    }
    if (!options.checkAbilityForInternal && !((_b = context.params) === null || _b === void 0 ? void 0 : _b.provider)) {
        return Promise.resolve(undefined);
    }
    if (options === null || options === void 0 ? void 0 : options.ability) {
        if (typeof options.ability === "function") {
            const ability = options.ability(context);
            return Promise.resolve(ability);
        }
        else {
            return Promise.resolve(options.ability);
        }
    }
    if (exports.getPersistedConfig(context, "ability")) {
        if (typeof options.ability === "function") {
            const ability = options.ability(context);
            return Promise.resolve(ability);
        }
        else {
            return Promise.resolve(options.ability);
        }
    }
    return Promise.resolve(undefined);
};
exports.getAbility = getAbility;
const move = (context, fromPath, toPath) => {
    const val = get_1.default(context, fromPath);
    if (val !== undefined) {
        unset_1.default(context, fromPath);
        set_1.default(context, toPath, val);
    }
    return val;
};
const throwUnlessCan = (ability, method, resource, modelName, actionOnForbidden) => {
    if (ability.cannot(method, resource)) {
        if (actionOnForbidden)
            actionOnForbidden();
        throw new errors_1.Forbidden(`You are not allowed to ${method} ${modelName}`);
    }
};
exports.throwUnlessCan = throwUnlessCan;
const handleConditionalSelect = (context, ability, method, modelName) => {
    var _a, _b, _c;
    if (!((_a = context.params) === null || _a === void 0 ? void 0 : _a.query) || !((_c = (_b = context.params) === null || _b === void 0 ? void 0 : _b.query) === null || _c === void 0 ? void 0 : _c.$select)) {
        return false;
    }
    const { $select } = context.params.query;
    const fields = getFieldsForConditions_1.default(ability, method, modelName);
    if (!fields.length) {
        return false;
    }
    const fieldsToAdd = fields.filter(field => !$select.includes(field));
    if (!fieldsToAdd.length) {
        return false;
    }
    exports.hide$select(context);
    context.params.query.$select = [...$select, ...fieldsToAdd];
    return true;
};
exports.handleConditionalSelect = handleConditionalSelect;
const checkMulti = (context, ability, modelName, options) => {
    const { method } = context;
    const currentIsMulti = feathers_utils_1.isMulti(context);
    if (!currentIsMulti) {
        return true;
    }
    if ((method === "find" && ability.can(method, modelName)) ||
        (ability.can(`${method}-multi`, modelName))) {
        return true;
    }
    if (options === null || options === void 0 ? void 0 : options.actionOnForbidden)
        options.actionOnForbidden();
    throw new errors_1.Forbidden(`You're not allowed to multi-${method} ${modelName}`);
};
exports.checkMulti = checkMulti;
const hide$select = (context) => {
    return move(context, "params.query.$select", "params.casl.$select");
};
exports.hide$select = hide$select;
const restore$select = (context) => {
    move(context, "params.casl.$select", "params.query.$select");
    return get_1.default(context, "params.query.$select");
};
exports.restore$select = restore$select;
const get$select = (context) => {
    return exports.getPersistedConfig(context, "$select");
};
exports.get$select = get$select;
const setPersistedConfig = (context, key, val) => {
    return set_1.default(context, `params.casl.${key}`, val);
};
exports.setPersistedConfig = setPersistedConfig;
const getPersistedConfig = (context, key) => {
    return get_1.default(context, `params.casl.${key}`);
};
exports.getPersistedConfig = getPersistedConfig;
